@@ -133,7 +133,7 @@ getPackageRecordsExternalSource <- function(pkgNames,
       # Construct the package record by hand -- generate the minimal
       # bits of the DESCRIPTION file, and infer the package record
       # from that.
-      pkg <- available[pkgName,]
+      pkg <- available[pkgName, ]
       df <- data.frame(
         Package = pkg[["Package"]],
         Version = pkg[["Version"]],
@@ -319,7 +319,7 @@ getPackageRecords <- function(pkgNames,
 # Return TRUE when the data frame for this package has the given RemoteType.
 hasRemoteType <- function(df, remoteType) {
   # Do not compare with 'identical'; RemoteType may be a factor.
-  return (!is.null(df$RemoteType) && df$RemoteType == remoteType)
+  return(!is.null(df$RemoteType) && df$RemoteType == remoteType)
 }
 
 # Reads a description file and attempts to infer where the package came from.
@@ -369,7 +369,8 @@ inferPackageRecord <- function(df, available = availablePackages()) {
       remote_username = as.character(df$RemoteUsername),
       remote_ref = as.character(df$RemoteRef),
       remote_sha = as.character(df$RemoteSha)),
-      c(remote_host = as.character(df$RemoteHost))
+      c(remote_host = as.character(df$RemoteHost)),
+      c(remote_subdir = as.character(df$RemoteSubdir))
     ), class = c('packageRecord', 'gitlab')))
   } else if (identical(as.character(df$Priority), 'base')) {
     # It's a base package!
@@ -455,7 +456,7 @@ getSourcePackageInfoImpl <- function(path) {
   ## For tarballs, we unzip them to a temporary directory and then read from there
   tempdir <- file.path(tempdir(), "packrat", path)
   if (endswith(path, "tar.gz")) {
-    untar(path, exdir = tempdir, tar = "internal")
+    untar(path, exdir = tempdir, tar = tar_binary())
     folderName <- list.files(tempdir, full.names = TRUE)[[1]]
   } else {
     folderName <- path
@@ -576,7 +577,7 @@ diffableRecord <- function(record) {
 # debug helper to print a package record. includes field names, type of value, and value.
 printPackageRecord <- function(name, record) {
   cat(name, "\n")
-  cat(paste(names(record), lapply(record,typeof), record, sep = ":", collapse = "\n"),"\n")
+  cat(paste(names(record), lapply(record, typeof), record, sep = ":", collapse = "\n"), "\n")
 }
 
 # states: NA (unchanged), remove, add, upgrade, downgrade, crossgrade
