@@ -2,7 +2,7 @@
 pkgSrcFilename <- function(pkgRecord) {
   if (identical(pkgRecord$source, "github"))
     paste(pkgRecord$gh_sha1, ".tar.gz", sep = "")
-  else if (identical(pkgRecord$source, "bitbucket"))
+  else if (pkgRecord$source %in% c("bitbucket", "gitlab"))
     paste(pkgRecord$remote_sha, ".tar.gz", sep = "")
   else
     paste(pkgRecord$name, "_", pkgRecord$version, ".tar.gz", sep = "")
@@ -23,21 +23,21 @@ isFromCranlikeRepo <- function(pkgRecord, repos) {
   if (!length(source))
     return(TRUE)
 
-  # for records that do declare a source, ensure it's not 'source', 'github' or 'bitbucket'.
+  # for records that do declare a source, ensure it's not 'source', 'github', 'bitbucket', or 'gitlab'.
   # in previous releases of packrat, we attempted to match the repository name
   # with one of the existing repositories; however, this caused issues in
   # certain environments (the names declared repositories in the lockfile, and
   # the the names of the active repositories in the R session, may not match)
-  !tolower(source) %in% c("source", "github", "bitbucket")
+  !tolower(source) %in% c("source", "github", "bitbucket", "gitlab")
 }
 
 # Given a package record and a database of packages, check to see if
 # the package version is current. NB: Assumes package is present in db.
 versionMatchesDb <- function(pkgRecord, db) {
   versionMatch <-
-    identical(pkgRecord$version, db[pkgRecord$name,"Version"])
+    identical(pkgRecord$version, db[pkgRecord$name, "Version"])
 
-  # For GitHub and Bitbucket, we also need to check that the SHA1 is identical
+  # For GitHub, Bitbucket, and Gitlab, we also need to check that the SHA1 is identical
   # (the source may be updated even if the version hasn't been bumped)
   if (versionMatch && identical(pkgRecord$source, "github")) {
     pkgDescFile <- system.file('DESCRIPTION', package = pkgRecord$name)
@@ -45,7 +45,7 @@ versionMatchesDb <- function(pkgRecord, db) {
       inferPackageRecord(as.data.frame(readDcf(pkgDescFile)))
     versionMatch <- identical(pkgRecord$gh_sha1,
                               installedPkgRecord$gh_sha1)
-  } else if (versionMatch && identical(pkgRecord$source, "bitbucket")) {
+  } else if (versionMatch && pkgRecord$source %in% c("gitlab", "bitbucket")) {
     pkgDescFile <- system.file('DESCRIPTION', package = pkgRecord$name)
     installedPkgRecord <-
       inferPackageRecord(as.data.frame(readDcf(pkgDescFile)))
@@ -57,6 +57,11 @@ versionMatchesDb <- function(pkgRecord, db) {
 
 # Given a package record, fetch the sources for the package and place them in
 # the source directory root given by sourceDir.
+# - Responsible for calling different download methods for different source
+#   locations (e.g. git hosting service, CRAN).
+# - Creates the path for the temporary destination file, named `srczip` at this
+#   level. It doesn't create the file itself — download functions do that — but
+#   handles its cleanup if it exists when the function exits.
 getSourceForPkgRecord <- function(pkgRecord,
                                   sourceDir,
                                   availablePkgs,
@@ -77,7 +82,8 @@ getSourceForPkgRecord <- function(pkgRecord,
   if (identical(pkgRecord$source, "unknown") && !quiet) {
     stop("No sources available for package ", pkgRecord$name, ". Packrat can ",
          "find sources for packages on CRAN-like repositories and packages ",
-         "installed using devtools::install_github or devtools::install_bitbucket. TODO: local repo")
+         "installed using devtools::install_github, devtools::install_gitlab",
+         "devtools::install_bitbucket. TODO: local repo")
   }
 
   # Create the directory in which to place this package's sources
@@ -94,7 +100,10 @@ getSourceForPkgRecord <- function(pkgRecord,
     message("Fetching sources for ", pkgRecord$name, " (", pkgRecord$version,
             ") ... ", appendLF = FALSE)
   }
+
   type <- pkgRecord$source
+  if (identical(type, "CustomCRANLikeRepository"))
+    type <- "CRAN"
 
   # If this is a local source path, compress the local sources rather than
   # trying to download from an external source
@@ -131,7 +140,7 @@ getSourceForPkgRecord <- function(pkgRecord,
         setwd(file.path(pkgRecord$source_path, ".."))
 
         tar(file.path(pkgSrcDir, pkgSrcFile), files = pkgRecord$name,
-            compression = "gzip", tar = "internal")
+            compression = "gzip", tar = tar_binary())
       }
     })
     type <- "local"
@@ -139,7 +148,7 @@ getSourceForPkgRecord <- function(pkgRecord,
 
     # Attempt to detect if this is the current version of a package
     # on a CRAN-like repository
-    currentVersion <- if (pkgRecord$name %in% rownames(availablePkgs))
+    currentVersion <- if (pkgRecord$name %in% availablePkgs[, "Package"])
       availablePkgs[pkgRecord$name, "Version"]
     else
       NA
@@ -180,8 +189,8 @@ getSourceForPkgRecord <- function(pkgRecord,
       # If the file wasn't saved to the destination directory (which can happen
       # if the repo is local--see documentation in download.packages), copy it
       # there now
-      if (!identical(fileLoc[1,2], file.path(pkgSrcDir, pkgSrcFile))) {
-        file.copy(fileLoc[1,2], pkgSrcDir)
+      if (!identical(fileLoc[1, 2], file.path(pkgSrcDir, pkgSrcFile))) {
+        file.copy(fileLoc[1, 2], pkgSrcDir)
       }
       type <- paste(type, "current")
     } else {
@@ -194,11 +203,9 @@ getSourceForPkgRecord <- function(pkgRecord,
           archiveUrl <- file.path(repo, "src/contrib/Archive",
                                   pkgRecord$name,
                                   pkgSrcFile)
-          if (!downloadWithRetries(archiveUrl,
-                                   destfile = file.path(pkgSrcDir, pkgSrcFile),
-                                   mode = "wb", quiet = TRUE)) {
-            stop("Failed to download package from URL:\n- ", shQuote(archiveUrl))
-          }
+          downloadWithRetries(archiveUrl,
+                              destfile = file.path(pkgSrcDir, pkgSrcFile),
+                              mode = "wb", quiet = TRUE)
           foundVersion <- TRUE
           type <- paste(type, "archived")
           break
@@ -219,194 +226,146 @@ getSourceForPkgRecord <- function(pkgRecord,
       }
     }
   } else if (identical(pkgRecord$source, "github")) {
-
-    # Prefer using https if possible. Note that 'wininet'
-    # can fail if attempting to download from an 'http'
-    # URL that redirects to an 'https' URL.
-    # https://github.com/rstudio/packrat/issues/269
-    method <- tryCatch(
-      secureDownloadMethod(),
-      error = function(e) "internal"
-    )
-
-    if (is.null(pkgRecord$remote_host) || !nzchar(pkgRecord$remote_host)) {
-      # Guard against packages installed with older versions of devtools
-      # (it's possible the associated package record will not contain a
-      # 'remote_host' entry)
-      protocol <- if (identical(method, "internal")) "http" else "https"
-      fmt <- "%s://api.github.com/repos/%s/%s/tarball/%s"
-      archiveUrl <- sprintf(fmt,
-                            protocol,
-                            pkgRecord$gh_username,
-                            pkgRecord$gh_repo,
-                            pkgRecord$gh_sha1)
-    } else {
-      # Prefer using the 'remote_host' entry as it allows for successfuly
-      # installation of packages available on private GitHub repositories
-      # (which will not use api.github.com)
-      fmt <- "%s/repos/%s/%s/tarball/%s"
-      archiveUrl <- sprintf(fmt,
-                            pkgRecord$remote_host,
-                            pkgRecord$remote_username,
-                            pkgRecord$remote_repo,
-                            pkgRecord$remote_sha)
-    }
+    archiveUrl <- githubArchiveUrl(pkgRecord)
 
     srczip <- tempfile(fileext = '.tar.gz')
-    on.exit({
-      if (file.exists(srczip))
-        unlink(srczip, recursive = TRUE)
-    }, add = TRUE)
+    on.exit(
+      unlink(srczip, recursive = TRUE),
+      add = TRUE
+    )
 
-    if (canUseGitHubDownloader()) {
-      status <- githubDownload(archiveUrl, srczip)
-      if (status) {
-        message("FAILED")
-        stop("Failed to download package from URL:\n- ", shQuote(archiveUrl))
-      }
-    } else {
-      success <- downloadWithRetries(archiveUrl, destfile = srczip, quiet = TRUE, mode = "wb")
-      if (!success) {
-        message("FAILED")
-        stop("Failed to download package from URL:\n- ", shQuote(archiveUrl))
-      }
-    }
+    tryCatch({
+      githubDownload(archiveUrl, srczip)
+    }, error = function(e) {
+      message("FAILED")
+      e$message <- sprintf("Failed to download package from GitHub URL: '%s'\n%s", archiveUrl, e$message)
+      stop(e)
+    })
 
-    local({
-      scratchDir <- tempfile()
-      on.exit({
-        if (file.exists(scratchDir))
-          unlink(scratchDir, recursive = TRUE)
-      })
-      # untar can emit noisy warnings (e.g. "skipping pax global extended
-      # headers"); hide those
-      suppressWarnings(untar(srczip, exdir = scratchDir, tar = "internal"))
-      # Find the base directory
-      basedir <- if (length(dir(scratchDir)) == 1)
-        file.path(scratchDir, dir(scratchDir))
-      else
-        scratchDir
-
-      if (!is.null(pkgRecord$gh_subdir))
-        basedir <- file.path(basedir, pkgRecord$gh_subdir)
-
-      if (!file.exists(file.path(basedir, 'DESCRIPTION'))) {
-        stop('No DESCRIPTION file was found in the archive for ', pkgRecord$name)
-      }
-
-      ghinfo <- as.data.frame(c(list(
-        GithubRepo = pkgRecord$gh_repo,
+    remote_info <- as.data.frame(
+      list(
+        RemoteType     = pkgRecord$source,
+        GithubRepo     = pkgRecord$gh_repo,
         GithubUsername = pkgRecord$gh_username,
-        GithubRef = pkgRecord$gh_ref,
-        GithubSHA1 = pkgRecord$gh_sha1)),
-        c(GithubSubdir = pkgRecord$gh_subdir)
-      )
-      appendToDcf(file.path(basedir, 'DESCRIPTION'), ghinfo)
+        GithubRef      = pkgRecord$gh_ref,
+        GithubSHA1     = pkgRecord$gh_sha1
+      ),
+      c(GithubSubdir   = pkgRecord$gh_subdir),
+      stringsAsFactors = FALSE
+    )
 
-      file.create(file.path(pkgSrcDir, pkgSrcFile))
-      dest <- normalizePath(file.path(pkgSrcDir, pkgSrcFile), winslash = '/')
+    dest <- normalizePath(file.path(pkgSrcDir, pkgSrcFile), winslash = "/", mustWork = FALSE)
 
-      # R's internal tar (which we use here for cross-platform consistency)
-      # emits warnings when there are > 100 characters in the path, due to the
-      # resulting incompatibility with older implementations of tar. This isn't
-      # relevant for our purposes, so suppress the warning.
-      in_dir(dirname(basedir),
-             suppressWarnings(tar(tarfile = dest, files = basename(basedir),
-                                  compression = 'gzip', tar = 'internal'))
+    tryCatch({
+      success <- appendRemoteInfoToDescription(
+        src = srczip,
+        dest = dest,
+        remote_info = remote_info
       )
+    }, error = function(e) {
+      e$message <- sprintf("Could not update 'DESCRIPTION' file for package %s:\n%s",
+           pkgRecord$name, e)
+      stop(e)
     })
 
     type <- "GitHub"
+
   } else if (identical(pkgRecord$source, "bitbucket")) {
-
-    # Prefer using https if possible. Note that 'wininet'
-    # can fail if attempting to download from an 'http'
-    # URL that redirects to an 'https' URL.
-    # https://github.com/rstudio/packrat/issues/269
-    method <- tryCatch(
-      secureDownloadMethod(),
-      error = function(e) "internal"
-    )
-
-    if (is.null(pkgRecord$remote_host) || !nzchar(pkgRecord$remote_host)) {
-      protocol <- if (identical(method, "internal")) "http" else "https"
-      pkgRecord$remote_host <- paste0(protocol, "://bitbucket.org")
-    }
-
-    fmt <- "%s/%s/%s/get/%s.tar.gz"
-    archiveUrl <- sprintf(fmt,
-                          pkgRecord$remote_host,
-                          pkgRecord$remote_username,
-                          pkgRecord$remote_repo,
-                          pkgRecord$remote_sha)
+    archiveUrl <- bitbucketArchiveUrl(pkgRecord)
 
     srczip <- tempfile(fileext = '.tar.gz')
-    on.exit({
-      if (file.exists(srczip))
-        unlink(srczip, recursive = TRUE)
-    }, add = TRUE)
+    on.exit(
+      unlink(srczip, recursive = TRUE),
+      add = TRUE
+    )
 
-    if (canUseBitbucketDownloader()) {
-      status <- bitbucketDownload(archiveUrl, srczip)
-      if (status) {
-        message("FAILED")
-        stop("Failed to download package from URL:\n- ", shQuote(archiveUrl))
-      }
-    } else {
-      success <- downloadWithRetries(archiveUrl, destfile = srczip, quiet = TRUE, mode = "wb")
-      if (!success) {
-        message("FAILED")
-        stop("Failed to download package from URL:\n- ", shQuote(archiveUrl))
-      }
-    }
+    tryCatch({
+      bitbucketDownload(archiveUrl, srczip)
+    }, error = function(e) {
+      message("FAILED")
+      e$message <- sprintf("Failed to download package from Bitbucket URL: '%s'\n%s", archiveUrl, e$message)
+      stop(e)
+    })
 
-    local({
-      scratchDir <- tempfile()
-      on.exit({
-        if (file.exists(scratchDir))
-          unlink(scratchDir, recursive = TRUE)
-      })
-      # untar can emit noisy warnings (e.g. "skipping pax global extended
-      # headers"); hide those
-      suppressWarnings(untar(srczip, exdir = scratchDir, tar = "internal"))
-      # Find the base directory
-      basedir <- if (length(dir(scratchDir)) == 1)
-        file.path(scratchDir, dir(scratchDir))
-      else
-        scratchDir
-
-      if (!is.null(pkgRecord$remote_subdir))
-        basedir <- file.path(basedir, pkgRecord$remote_subdir)
-
-      if (!file.exists(file.path(basedir, 'DESCRIPTION'))) {
-        stop('No DESCRIPTION file was found in the archive for ', pkgRecord$name)
-      }
-
-      remote_info <- as.data.frame(c(list(
+    # Modify remote info, move modified package to new location
+    remote_info <- as.data.frame(
+      list(
+        RemoteType = pkgRecord$source,
+        RemoteHost = pkgRecord$remote_host,
         RemoteRepo = pkgRecord$remote_repo,
         RemoteUsername = pkgRecord$remote_username,
         RemoteRef = pkgRecord$remote_ref,
-        RemoteSha = pkgRecord$remote_sha)),
-        c(RemoteSubdir = pkgRecord$remote_subdir)
-      )
-      appendToDcf(file.path(basedir, 'DESCRIPTION'), remote_info)
+        RemoteSha = pkgRecord$remote_sha
+      ),
+      c(RemoteSubdir = pkgRecord$remote_subdir),
+      stringsAsFactors = FALSE
+    )
 
-      file.create(file.path(pkgSrcDir, pkgSrcFile))
-      dest <- normalizePath(file.path(pkgSrcDir, pkgSrcFile), winslash = '/')
+    dest <- normalizePath(file.path(pkgSrcDir, pkgSrcFile), winslash = "/", mustWork = FALSE)
 
-      # R's internal tar (which we use here for cross-platform consistency)
-      # emits warnings when there are > 100 characters in the path, due to the
-      # resulting incompatibility with older implementations of tar. This isn't
-      # relevant for our purposes, so suppress the warning.
-      in_dir(dirname(basedir),
-             suppressWarnings(tar(tarfile = dest, files = basename(basedir),
-                                  compression = 'gzip', tar = 'internal'))
+    tryCatch({
+      success <- appendRemoteInfoToDescription(
+        src = srczip,
+        dest = dest,
+        remote_info = remote_info
       )
+    }, error = function(e) {
+      e$message <- sprintf("Could not update 'DESCRIPTION' file for package %s:\n%s",
+           pkgRecord$name, e)
+      stop(e)
     })
 
     type <- "Bitbucket"
+
+
+  } else if (identical(pkgRecord$source, "gitlab")) {
+    archiveUrl <- gitlabArchiveUrl(pkgRecord)
+
+    srczip <- tempfile(fileext = '.tar.gz')
+    on.exit(
+      unlink(srczip, recursive = TRUE),
+      add = TRUE
+    )
+
+    tryCatch({
+      gitlabDownload(archiveUrl, srczip)
+    }, error = function(e) {
+      message("FAILED")
+      e$message <- sprintf("Failed to download package from GitLab URL: '%s'\n%s", archiveUrl, e$message)
+      stop(e)
+    })
+
+    # Modify remote info, move modified package to new location
+    remote_info <- as.data.frame(
+      list(
+        RemoteType = pkgRecord$source,
+        RemoteHost = pkgRecord$remote_host,
+        RemoteRepo = pkgRecord$remote_repo,
+        RemoteUsername = pkgRecord$remote_username,
+        RemoteRef = pkgRecord$remote_ref,
+        RemoteSha = pkgRecord$remote_sha
+      ),
+      c(RemoteSubdir = pkgRecord$remote_subdir),
+      stringsAsFactors = FALSE
+    )
+
+    dest <- normalizePath(file.path(pkgSrcDir, pkgSrcFile), winslash = "/", mustWork = FALSE)
+
+    tryCatch({
+      success <- appendRemoteInfoToDescription(
+        src = srczip,
+        dest = dest,
+        remote_info = remote_info
+      )
+    }, error = function(e) {
+      e$message <- sprintf("Could not update 'DESCRIPTION' file for package %s:\n%s",
+           pkgRecord$name, e)
+      stop(e)
+    })
+
+    type <- "GitLab"
   }
-  if (!quiet) {
+  if (!quiet) { # TODO: Does turning on (quiet) prevent it from failing on error here
     if (file.exists(file.path(pkgSrcDir, pkgSrcFile))) {
       message("OK (", type, ")")
     } else {
@@ -480,7 +439,7 @@ annotatePkgs <- function(pkgNames, project, lib = libDir(project)) {
 
 # Takes a vector of package names, and returns a logical vector that indicates
 # whether the package was not installed by packrat.
-installedByPackrat <- function(pkgNames, lib.loc, default=NA) {
+installedByPackrat <- function(pkgNames, lib.loc, default = NA) {
   # Can't use installed.packages(fields='InstallAgent') here because it uses
   # Meta/package.rds, not the DESCRIPTION file, and we only record this info in
   # the DESCRIPTION file.
@@ -495,10 +454,8 @@ installedByPackrat <- function(pkgNames, lib.loc, default=NA) {
   })))
 }
 
-# Removes one or more packages from the app's private library and cached
-# sources.
+# Removes one or more packages from the app's private library.
 removePkgs <- function(project, pkgNames, lib.loc = libDir(project)) {
-  unlink(file.path(srcDir(project), pkgNames), recursive = TRUE)
   remove.packages(pkgNames, lib.loc)
   pkgNames
 }
@@ -573,10 +530,11 @@ installPkg <- function(pkgRecord,
 
   # Try downloading a binary (when appropriate).
   if (!(copiedFromCache || copiedFromUntrustedCache) &&
-        hasBinaryRepositories() &&
-        isFromCranlikeRepo(pkgRecord, repos) &&
-        pkgRecord$name %in% rownames(availablePackagesBinary(repos = repos)) &&
-        versionMatchesDb(pkgRecord, availablePackagesBinary(repos = repos)))
+      hasBinaryRepositories() &&
+      binaryRepositoriesEnabled() &&
+      isFromCranlikeRepo(pkgRecord, repos) &&
+      pkgRecord$name %in% availablePackagesBinary(repos = repos)[, "Package"] &&
+      versionMatchesDb(pkgRecord, availablePackagesBinary(repos = repos)))
   {
     tempdir <- tempdir()
     tryCatch({
@@ -608,7 +566,7 @@ installPkg <- function(pkgRecord,
   }
 
   if (is.null(pkgSrc)) {
-    # When installing from github/bitbucket or an older version, use the cached source
+    # When installing from github/bitbucket/gitlab or an older version, use the cached source
     # tarball or zip created in snapshotSources
     pkgSrc <- file.path(srcDir(project), pkgRecord$name,
                         pkgSrcFilename(pkgRecord))
@@ -631,6 +589,16 @@ installPkg <- function(pkgRecord,
       }
     }
 
+    # Infer package type; note that RSPM may deliver binary packages
+    # in archives with .tar.gz extension.
+    pkgType <- tryCatch(
+      archivePackageType(pkgSrc, quiet = TRUE),
+      error = identity
+    )
+
+    if (identical(pkgType, "binary"))
+      type <- "downloaded binary"
+
     local({
       # devtools does not install to any libraries other than the default, so
       # if the library we wish to install to is not the default, set as the
@@ -710,7 +678,7 @@ playActions <- function(pkgRecords, actions, repos, project, lib) {
       # Changing package type or version: Remove the old one now (we'll write
       # a new one in a moment)
       message("Replacing ", pkgRecord$name, " (", action, " ",
-              installedPkgs[pkgRecord$name,"Version"], " to ",
+              installedPkgs[pkgRecord$name, "Version"], " to ",
               pkgRecord$version, ") ... ", appendLF = FALSE)
       removePkgs(project, pkgRecord$name, lib)
     } else if (identical(action, "add")) {
@@ -880,7 +848,7 @@ detachPackageForInstallationIfNecessary <- function(pkg) {
 }
 
 discoverUntrustedPackages <- function(srcDir) {
-  if (is.na(Sys.getenv("RSTUDIO_CONNECT", unset = NA)))
+  if (is.na(Sys.getenv("POSIT_CONNECT", unset = NA)))
     return()
 
   # set the 'packrat.untrusted.packages' option if
@@ -926,3 +894,118 @@ overlaySourcePackages <- function(srcDir, overlayDir = NULL) {
     file.exists(target)
   })
 }
+
+archivePackageType <- function(path, quiet = FALSE, default = "source") {
+
+  info <- file.info(path, extra_cols = FALSE)
+  if (is.na(info$isdir))
+    stopf("no package at path %s", shQuote(path, type = "cmd"))
+
+  # for directories, check for Meta
+  if (info$isdir) {
+    hasmeta <- file.exists(file.path(path, "Meta"))
+    type <- if (hasmeta) "binary" else "source"
+    return(type)
+  }
+
+  # otherwise, guess based on contents of package
+  methods <- list(
+    tar = function(path) untar(tarfile = path, list = TRUE),
+    zip = function(path) unzip(zipfile = path, list = TRUE)$Name
+  )
+
+  # try zip first for files ending with '.zip'
+  # (but attempt to be robust against mis-named files)
+  if (endswith(path, ".zip"))
+    methods <- methods[c("zip", "tar")]
+
+  for (method in methods) {
+
+    files <- tryCatch(method(path), error = identity)
+    if (inherits(files, "error"))
+      next
+
+    hasmeta <- any(grepl("^[^/]+/Meta/?$", files))
+    type <- if (hasmeta) "binary" else "source"
+    return(type)
+
+  }
+
+  if (!quiet) {
+    fmt <- "failed to determine type of package '%s'; assuming source"
+    warningf(fmt, shQuote(path, type = "cmd"))
+  }
+
+  default
+
+}
+
+
+# Decompresses the archive passed to `src`. Appends `remote_info` to the
+# DESCRIPTION file. Recompresses the file passed to `dest`, which must be
+# a `.tar.gz`. Returns TRUE if successful.
+appendRemoteInfoToDescription <- function(src, dest, remote_info) {
+  # We expect `dest` to end with `".tar.gz"`.
+  if (!grepl(".tar.gz$", dest)) {
+    stop("Destination path for source archive must end in '.tar.gz'.")
+  }
+
+  # Extract the package to a temporary dir so that we can modify the
+  # `DESCRIPTION` with the remote info.
+  scratchDir <- tempfile()
+  on.exit({
+    if (file.exists(scratchDir))
+      unlink(scratchDir, recursive = TRUE)
+  })
+  # untar can emit noisy warnings (e.g. "skipping pax global extended
+  # headers"); hide those
+  suppressWarnings(untar(src, exdir = scratchDir, tar = tar_binary()))
+
+  # Determine the untarred base directory. We're looking to see if the untarred
+  # directory contains only a single directory and if so, we treat that as our
+  # base directory.
+  if (length(dir(scratchDir)) == 1 &&
+      is.directory(file.path(scratchDir, dir(scratchDir)))) {
+    basedir <- file.path(scratchDir, dir(scratchDir))
+  } else {
+    basedir <- scratchDir
+  }
+
+  # Determine the true package root
+  if (remote_info$RemoteType == "github") {
+    remote_subdir <- remote_info$GithubSubdir
+  } else {
+    remote_subdir <- remote_info$RemoteSubdir
+  }
+  if (length(remote_subdir) > 0) {
+    basedir <- file.path(basedir, remote_subdir)
+  }
+
+  if (!file.exists(file.path(basedir, "DESCRIPTION"))) {
+    # This error may indicate a malformed package, or an unexpected directory
+    # structure inside the tarball.
+    stop("Could not locate DESCRIPTION file in package archive.")
+  }
+
+  # Do what we came here to do.
+  appendToDcf(file.path(basedir, "DESCRIPTION"), remote_info)
+
+  # Now we can recompress the file to wherever we've been told to do so.
+  # R's internal tar (which we use here for cross-platform consistency)
+  # emits warnings when there are > 100 characters in the path, due to the
+  # resulting incompatibility with older implementations of tar. This isn't
+  # relevant for our purposes, so suppress the warning.
+  # tryCatch here so we can unlink the file if tar fails.
+  tryCatch(
+    in_dir(dirname(basedir),
+            suppressWarnings(tar(tarfile = dest, files = basename(basedir),
+                                compression = "gzip", tar = tar_binary()))
+    ),
+    error = function(e) {
+      unlink(dest)
+      stop(e)
+    }
+  )
+
+  return(TRUE)
+}
